/* Copyright (C) 1999-2002 Stacey L. Keast <slik@telusplanet.net>
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software 
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "bbrb.h"

/* User interface */

void
createUI (void)
{

  /*--- Create top level window
  -----------------------------*/

  GString *gsWindowTitle = g_string_new (PACKAGE);
  g_string_append (gsWindowTitle, " v");
  g_string_append (gsWindowTitle, VERSION);
  g_string_append (gsWindowTitle, " Copyright (C) 1999-2002 Stacey Keast");
  wpTopLevelWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (wpTopLevelWindow), gsWindowTitle->str);
  g_string_free (gsWindowTitle, TRUE);
  gtk_window_set_policy (GTK_WINDOW (wpTopLevelWindow), DISALLOW_SHRINK,
			 DISALLOW_GROW, DISALLOW_AUTOSHRINK);
  gtk_container_set_border_width (GTK_CONTAINER (wpTopLevelWindow), BORDER);

  /*
   * Signal Handlers for wpTopLevelWindow 
   */
  gtk_signal_connect (GTK_OBJECT (wpTopLevelWindow), "destroy",
		      GTK_SIGNAL_FUNC (destroy_CB), NULL);
  gtk_signal_connect (GTK_OBJECT (wpTopLevelWindow), "delete_event",
		      GTK_SIGNAL_FUNC (delete_event_CB), NULL);

  /*--- Create layout containers
  ------------------------------*/

  wpMainVBox = gtk_vbox_new (NON_HOMOGENEOUS, NO_SPACING);

  /*--- TopHBox - list, list buttons and pixmap
  ---------------------------------------------*/

  /*
   * TopLeftVBox 
   */
  BoxTopH = gtk_hbox_new (NON_HOMOGENEOUS, NO_SPACING);
  BoxTopLeftV = gtk_vbox_new (NON_HOMOGENEOUS, NO_SPACING);
  wpPictureListScrolledWindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW
				  (wpPictureListScrolledWindow),
				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_widget_set_usize (wpPictureListScrolledWindow, 250, 150);
  BoxTopMidV = gtk_vbox_new (NON_HOMOGENEOUS, NO_SPACING);
  BoxTopRightV = gtk_vbox_new (NON_HOMOGENEOUS, NO_SPACING);
  gtk_widget_set_usize (BoxTopRightV, 158, 150);

  /*
   * initialize tooltips 
   */
  tooltips = gtk_tooltips_new ();

  /*
   * TopMidVBox 
   */
  wpAddButton = gtk_button_new_with_label ("Add");
  gtk_tooltips_set_tip (tooltips, wpAddButton,
			"Add a picutre to the list at left.", NULL);
  wpRemoveButton = gtk_button_new_with_label ("Remove");
  gtk_tooltips_set_tip (tooltips, wpRemoveButton,
			"Remove the selected picture from the list.", NULL);
  wpRelabelButton = gtk_button_new_with_label ("Relabel");
  gtk_tooltips_set_tip (tooltips, wpRelabelButton,
			"Assign a different label (not file name) to a selected picture in list.",
			NULL);
  wpClearButton = gtk_button_new_with_label ("Clear List");
  gtk_tooltips_set_tip (tooltips, wpClearButton,
			"Clear the list at left.", NULL);
  wpSortButton = gtk_button_new_with_label ("Sort List");
  gtk_tooltips_set_tip (tooltips, wpSortButton,
			"Sort the list of labels in alphanumeric order.",
			NULL);
  wpRandomizeButton = gtk_toggle_button_new_with_label ("Random");
  gtk_tooltips_set_tip (tooltips, wpRandomizeButton,
			"Toggle this button and hit \"Set Root Image\" to have a random picture from the list shown when "
			"Blackbox starts.  To get rid of randomness, you must select a picture and click "
			"\"Set Root Image\" or select an image from the root menu.",
			NULL);

  if (bRandomize)
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpRandomizeButton),
				    TRUE);
      gtk_widget_set_sensitive (wpRandomizeButton, FALSE);
    }

  /*
   * start insensitive until something picked in list 
   */
  gtk_widget_set_state (wpRemoveButton, GTK_STATE_INSENSITIVE);
  gtk_widget_set_state (wpRelabelButton, GTK_STATE_INSENSITIVE);
  if (giRows < 2)
    {
      gtk_widget_set_state (wpSortButton, GTK_STATE_INSENSITIVE);
      gtk_widget_set_state (wpRandomizeButton, GTK_STATE_INSENSITIVE);
      bRandomize = FALSE;
    }
  if (giRows < 1)
    gtk_widget_set_state (wpClearButton, GTK_STATE_INSENSITIVE);

  /*
   * TopRightVBox - pixmap 
   */
  wpPixmap = bbrb_show_thumb ("DEFAULTPIXMAP", wpPixmap, TRUE);

  /*
   * MidBox - display options 
   */
  BoxMidV = gtk_vbox_new (NON_HOMOGENEOUS, NO_SPACING);
  BoxImageSettersH = gtk_hbox_new (NON_HOMOGENEOUS, SPACING);
  BoxXVV = gtk_vbox_new (NON_HOMOGENEOUS, NO_SPACING);

  /*
   * image setters 
   */
  wpImageSetterButtonGroup = NULL;
  if (bHaveXV)
    {
      wpXV = gtk_radio_button_new_with_label (wpImageSetterButtonGroup, "xv");
      wpImageSetterButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpXV));
    }
  if (bHaveEsetroot)
    {
      wpEsetroot =
	gtk_radio_button_new_with_label (wpImageSetterButtonGroup,
					 "Esetroot");
      wpImageSetterButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpEsetroot));
    }

  /*
   * Esetroot 
   */

  if (bHaveEsetroot)
    {
      FrameEsetroot = gtk_frame_new ("Options");
      BoxEsetrootV = gtk_vbox_new (NON_HOMOGENEOUS, NO_SPACING);
      wpEsetrootRadioButtonGroup = NULL;
      wpEsetrootTile =
	gtk_radio_button_new_with_label (wpEsetrootRadioButtonGroup, "Tile");
      wpEsetrootRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpEsetrootTile));
      wpEsetrootCentered =
	gtk_radio_button_new_with_label (wpEsetrootRadioButtonGroup,
					 "Centered");
      wpEsetrootRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpEsetrootCentered));
      wpEsetrootMax =
	gtk_radio_button_new_with_label (wpEsetrootRadioButtonGroup,
					 "Stretch Maximum");
      wpEsetrootRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpEsetrootMax));
      wpEsetrootMaxAspect =
	gtk_radio_button_new_with_label (wpEsetrootRadioButtonGroup,
					 "Maximum Preserving Aspect");
      wpEsetrootRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpEsetrootMaxAspect));
    }

  /*
   * xv 
   */

  if (bHaveXV)
    {
		BoxXVFillMiscH = gtk_hbox_new (NON_HOMOGENEOUS, NO_SPACING);
      /*
       * Fill - Frame 
       */
      FrameXVFill = gtk_frame_new ("Stretching Options");
      BoxXVFillV = gtk_vbox_new (NON_HOMOGENEOUS, NO_SPACING);
      /*
       * initialize group 
       */
      wpFillRadioButtonGroup = NULL;
      /*
       * make radio buttons whilst adding them to a group 
       */
      wpFillNo =
	gtk_radio_button_new_with_label (wpFillRadioButtonGroup,
					 "Don't Stretch");
      wpFillRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpFillNo));
      wpFillMax =
	gtk_radio_button_new_with_label (wpFillRadioButtonGroup, "Maximum");
      wpFillRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpFillMax));
      wpFillMaxAspect =
	gtk_radio_button_new_with_label (wpFillRadioButtonGroup,
					 "Maximum Preserving Aspect");
      wpFillRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpFillMaxAspect));

      /*
       * Tile - Frame 
       */
      FrameXVTile = gtk_frame_new ("Tiling Options");
      BoxXVTileV1 = gtk_vbox_new (NON_HOMOGENEOUS, NO_SPACING);
	  BoxXVTileV2 = gtk_vbox_new (NON_HOMOGENEOUS, NO_SPACING);
	  BoxXVTileH = gtk_hbox_new (NON_HOMOGENEOUS, NO_SPACING);
      /*
       * initialize group 
       */
      wpTileRadioButtonGroup = NULL;
      /*
       * make radio buttons whilst adding them to a group 
       */
      wpTile =
	gtk_radio_button_new_with_label (wpTileRadioButtonGroup, "Tiled");
      wpTileRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpTile));
      wpTileInteger =
	gtk_radio_button_new_with_label (wpTileRadioButtonGroup,
					 "Integer Tiling");
      wpTileRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpTileInteger));
      wpTileMirrored =
	gtk_radio_button_new_with_label (wpTileRadioButtonGroup,
					 "Mirrored Tiling");
      wpTileRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpTileMirrored));
      wpTileIntegerMirrored =
	gtk_radio_button_new_with_label (wpTileRadioButtonGroup,
					 "Integer Mirrored Tiling");
      wpTileRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpTileIntegerMirrored));
      wpTileCentered =
	gtk_radio_button_new_with_label (wpTileRadioButtonGroup,
					 "Centered Tiling");
      wpTileRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpTileCentered));
      wpTileCenteredSolidBackground =
	gtk_radio_button_new_with_label (wpTileRadioButtonGroup,
					 "Centered on Solid Background");
      wpTileRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON
				(wpTileCenteredSolidBackground));
      wpTileSymmetrical =
	gtk_radio_button_new_with_label (wpTileRadioButtonGroup,
					 "Symmetrical Tiling");
      wpTileRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpTileSymmetrical));
      wpTileSymmetricalMirrored =
	gtk_radio_button_new_with_label (wpTileRadioButtonGroup,
					 "Symmetrical Mirrored Tiling");
      wpTileRadioButtonGroup =
	gtk_radio_button_group (GTK_RADIO_BUTTON (wpTileSymmetricalMirrored));

      /*
       * Misc - Frame 
       */
      FrameXVMisc = gtk_frame_new ("Misc. Options");
      BoxXVMiscV = gtk_vbox_new (NON_HOMOGENEOUS, NO_SPACING);
      /*
       * make checkbuttons 
       */
      wpMiscReverseVideoCheckButton =
	gtk_check_button_new_with_label ("Reverse Video");
      wpMiscHorizontalFlipCheckButton =
	gtk_check_button_new_with_label ("Flip Image Horizontally");
      wpMiscVerticalFlipCheckButton =
	gtk_check_button_new_with_label ("Flip Image Vertically");
      wpMiscSmoothCheckButton =
	gtk_check_button_new_with_label ("Smooth Image");
    }

  /*
   * widget to add space between bottom buttons and bottom frame (misc) 
   */
  wpSeparator = gtk_hseparator_new ();

  /*
   * BotHBox - command buttons 
   */
  BoxBotH = gtk_hbox_new (HOMOGENEOUS, NO_SPACING);

  /*--- Create bottom buttons
  ---------------------------*/

  wpUpdateButton = gtk_button_new_with_label ("Update Menu");
  gtk_tooltips_set_tip (tooltips, wpUpdateButton, "Save changes.", NULL);
  wpSetRootButton = gtk_button_new_with_label ("Set Root Image");
  gtk_tooltips_set_tip (tooltips, wpSetRootButton,
			"Change the desktop background to selected image.",
			NULL);
  wpCancelButton = gtk_button_new_with_label ("Exit");
  gtk_tooltips_set_tip (tooltips, wpCancelButton,
			"Exit bbrb.",
			NULL);

  /*
   * set Update and Set buttons INSENSITIVE 
   */

  gtk_widget_set_state (wpUpdateButton, GTK_STATE_INSENSITIVE);
  if (!bRandomize)
    gtk_widget_set_state (wpSetRootButton, GTK_STATE_INSENSITIVE);

  /*
   * Signal Connections for top image setter buttons 
   */
  if (bHaveXV)
    gtk_signal_connect (GTK_OBJECT (wpXV), "clicked",
			GTK_SIGNAL_FUNC (bbrb_show_xv), NULL);
  if (bHaveEsetroot)
    gtk_signal_connect (GTK_OBJECT (wpEsetroot), "clicked",
			GTK_SIGNAL_FUNC (bbrb_show_Esetroot), NULL);

  /*
   * Signal Connections for top mid buttons 
   */
  gtk_signal_connect (GTK_OBJECT (wpAddButton), "clicked",
		      GTK_SIGNAL_FUNC (bbrb_list_add), NULL);
  gtk_signal_connect (GTK_OBJECT (wpRemoveButton), "clicked",
		      GTK_SIGNAL_FUNC (bbrb_list_remove), NULL);
  gtk_signal_connect (GTK_OBJECT (wpRelabelButton), "clicked",
		      GTK_SIGNAL_FUNC (bbrb_set_label), NULL);
  gtk_signal_connect (GTK_OBJECT (wpClearButton), "clicked",
		      GTK_SIGNAL_FUNC (bbrb_list_clear), NULL);
  gtk_signal_connect (GTK_OBJECT (wpSortButton), "clicked",
		      GTK_SIGNAL_FUNC (bbrb_list_sort), NULL);
  gtk_signal_connect (GTK_OBJECT (wpRandomizeButton), "toggled",
		      GTK_SIGNAL_FUNC (bbrb_randomize_toggled), NULL);

  /*
   * Signal Connections for option buttons 
   */
  /*
   * Esetroot 
   */
  if (bHaveEsetroot)
    {
      gtk_signal_connect (GTK_OBJECT (wpEsetrootTile), "toggled",
			  GTK_SIGNAL_FUNC (bbrb_update_item), NULL);
      gtk_signal_connect (GTK_OBJECT (wpEsetrootCentered),
			  "toggled", GTK_SIGNAL_FUNC (bbrb_update_item),
			  NULL);
      gtk_signal_connect (GTK_OBJECT (wpEsetrootMax), "toggled",
			  GTK_SIGNAL_FUNC (bbrb_update_item), NULL);
      gtk_signal_connect (GTK_OBJECT (wpEsetrootMaxAspect), "toggled",
			  GTK_SIGNAL_FUNC (bbrb_update_item), NULL);
    }
  /*
   * xv 
   */
  if (bHaveXV)
    {
      gtk_signal_connect (GTK_OBJECT (wpFillNo), "toggled",
			  GTK_SIGNAL_FUNC (bbrb_update_item), NULL);
      gtk_signal_connect (GTK_OBJECT (wpFillMax), "toggled",
			  GTK_SIGNAL_FUNC (bbrb_update_item), NULL);
      gtk_signal_connect (GTK_OBJECT (wpFillMaxAspect), "toggled",
			  GTK_SIGNAL_FUNC (bbrb_update_item), NULL);
      gtk_signal_connect (GTK_OBJECT (wpTile), "toggled",
			  GTK_SIGNAL_FUNC (bbrb_update_item), NULL);
      gtk_signal_connect (GTK_OBJECT (wpTileInteger), "toggled",
			  GTK_SIGNAL_FUNC (bbrb_update_item), NULL);
      gtk_signal_connect (GTK_OBJECT (wpTileMirrored), "toggled",
			  GTK_SIGNAL_FUNC (bbrb_update_item), NULL);
      gtk_signal_connect (GTK_OBJECT (wpTileIntegerMirrored),
			  "toggled", GTK_SIGNAL_FUNC (bbrb_update_item),
			  NULL);
      gtk_signal_connect (GTK_OBJECT (wpTileCentered), "toggled",
			  GTK_SIGNAL_FUNC (bbrb_update_item), NULL);
      gtk_signal_connect (GTK_OBJECT (wpTileCenteredSolidBackground),
			  "toggled", GTK_SIGNAL_FUNC (bbrb_update_item),
			  NULL);
      gtk_signal_connect (GTK_OBJECT (wpTileSymmetrical), "toggled",
			  GTK_SIGNAL_FUNC (bbrb_update_item), NULL);
      gtk_signal_connect (GTK_OBJECT (wpTileSymmetricalMirrored), "toggled",
			  GTK_SIGNAL_FUNC (bbrb_update_item), NULL);
      gtk_signal_connect (GTK_OBJECT (wpMiscHorizontalFlipCheckButton),
			  "toggled", GTK_SIGNAL_FUNC (bbrb_update_item),
			  NULL);
      gtk_signal_connect (GTK_OBJECT (wpMiscVerticalFlipCheckButton),
			  "toggled", GTK_SIGNAL_FUNC (bbrb_update_item),
			  NULL);
      gtk_signal_connect (GTK_OBJECT (wpMiscSmoothCheckButton), "toggled",
			  GTK_SIGNAL_FUNC (bbrb_update_item), NULL);
      gtk_signal_connect (GTK_OBJECT (wpMiscReverseVideoCheckButton),
			  "toggled", GTK_SIGNAL_FUNC (bbrb_update_item),
			  NULL);
    }

  /*
   * Signal Connections for bottom buttons 
   */
  gtk_signal_connect (GTK_OBJECT (wpCancelButton), "clicked",
		      GTK_SIGNAL_FUNC (destroy_CB), NULL);
  gtk_signal_connect (GTK_OBJECT (wpSetRootButton), "clicked",
		      GTK_SIGNAL_FUNC (bbrb_set_root), NULL);
  gtk_signal_connect (GTK_OBJECT (wpUpdateButton), "clicked",
		      GTK_SIGNAL_FUNC (bbrb_update), NULL);

  /*
   * List Signals 
   */
  gtk_signal_connect (GTK_OBJECT (wpPictureList), "select_row",
		      GTK_SIGNAL_FUNC (listitem_selected), NULL);
  gtk_signal_connect (GTK_OBJECT (wpPictureList), "unselect_row",
		      GTK_SIGNAL_FUNC (listitem_deselected), NULL);

  /*--- Packing
  -------------*/

  /*
   * Pack scrolled window with list - top left 
   */
  gtk_container_set_border_width (GTK_CONTAINER (BoxTopLeftV), BORDER);
  gtk_container_add (GTK_CONTAINER (wpPictureListScrolledWindow),
		     wpPictureList);
  gtk_container_add (GTK_CONTAINER (BoxTopLeftV),
		     wpPictureListScrolledWindow);

  /*
   * Pack buttons - topmid 
   */
  gtk_box_pack_start (GTK_BOX (BoxTopMidV), wpAddButton, EXPAND_NO,
		      FILL_NO, NO_PADDING);
  gtk_box_pack_start (GTK_BOX (BoxTopMidV), wpRemoveButton, EXPAND_NO,
		      FILL_NO, NO_PADDING);
  gtk_box_pack_start (GTK_BOX (BoxTopMidV), wpRelabelButton,
		      EXPAND_NO, FILL_NO, NO_PADDING);
  gtk_box_pack_end (GTK_BOX (BoxTopMidV), wpRandomizeButton,
		    EXPAND_NO, FILL_NO, NO_PADDING);
  gtk_box_pack_end (GTK_BOX (BoxTopMidV), wpSortButton, EXPAND_NO,
		    FILL_NO, NO_PADDING);
  gtk_box_pack_end (GTK_BOX (BoxTopMidV), wpClearButton, EXPAND_NO,
		    FILL_NO, NO_PADDING);
  /*
   * Pack pixmap - top right 
   */
  gtk_container_set_border_width (GTK_CONTAINER (BoxTopRightV), BORDER);
  gtk_container_add (GTK_CONTAINER (BoxTopRightV), wpPixmap);

  /*
   * Pack buttons - bottom 
   */
  gtk_box_pack_start (GTK_BOX (BoxBotH), wpUpdateButton, EXPAND_YES,
		      FILL_YES, NO_PADDING);
  gtk_box_pack_start (GTK_BOX (BoxBotH), wpSetRootButton, EXPAND_YES,
		      FILL_YES, NO_PADDING);
  gtk_box_pack_start (GTK_BOX (BoxBotH), wpCancelButton, EXPAND_YES,
		      FILL_YES, NO_PADDING);

  /*
   * Add box wigets to parent containers 
   */
  gtk_container_add (GTK_CONTAINER (BoxTopH), BoxTopLeftV);
  gtk_container_add (GTK_CONTAINER (BoxTopH), BoxTopMidV);
  gtk_container_add (GTK_CONTAINER (BoxTopH), BoxTopRightV);

  /*
   * Image Setters 
   */
  if (giNumberOfImageSetters != 1)
    {
      if (bHaveXV)
	gtk_box_pack_start (GTK_BOX (BoxImageSettersH), wpXV,
			    EXPAND_NO, FILL_NO, NO_PADDING);
      if (bHaveEsetroot)
	gtk_box_pack_start (GTK_BOX (BoxImageSettersH), wpEsetroot,
			    EXPAND_NO, FILL_NO, NO_PADDING);
    }

  /*
   * Esetroot 
   */
  if (bHaveEsetroot)
    {
      gtk_box_pack_start (GTK_BOX (BoxEsetrootV), wpEsetrootTile,
			  EXPAND_NO, FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxEsetrootV),
			  wpEsetrootCentered, EXPAND_NO, FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxEsetrootV), wpEsetrootMax,
			  EXPAND_NO, FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxEsetrootV),
			  wpEsetrootMaxAspect, EXPAND_NO, FILL_NO,
			  NO_PADDING);
      gtk_frame_set_label_align (GTK_FRAME (FrameEsetroot), 0.5, 0);
      gtk_container_add (GTK_CONTAINER (FrameEsetroot), BoxEsetrootV);
    }

  /*
   * xv 
   */
  if (bHaveXV)
  {
      /*
       * Fill Frame 
       */
      gtk_box_pack_start (GTK_BOX (BoxXVFillV), wpFillNo,
			  EXPAND_NO, FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxXVFillV), wpFillMax,
			  EXPAND_NO, FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxXVFillV), wpFillMaxAspect,
			  EXPAND_NO, FILL_NO, NO_PADDING);
      gtk_frame_set_label_align (GTK_FRAME (FrameXVFill), 0.5, 0);
      gtk_container_add (GTK_CONTAINER (FrameXVFill), BoxXVFillV);
	  gtk_container_add (GTK_CONTAINER (BoxXVFillMiscH),FrameXVFill);
      gtk_container_add (GTK_CONTAINER (BoxXVV), BoxXVFillMiscH);
	  
      /*
       * Tile Frame 
       */
      gtk_box_pack_start (GTK_BOX (BoxXVTileV1),
			  wpTileCenteredSolidBackground, EXPAND_NO,
			  FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxXVTileV2), wpTile, EXPAND_NO,
			  FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxXVTileV1), wpTileInteger,
			  EXPAND_NO, FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxXVTileV1), wpTileMirrored,
			  EXPAND_NO, FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxXVTileV1),
			  wpTileIntegerMirrored, EXPAND_NO, FILL_NO,
			  NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxXVTileV2), wpTileCentered, EXPAND_NO,
			  FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxXVTileV2), wpTileSymmetrical,
			  EXPAND_NO, FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxXVTileV2), wpTileSymmetricalMirrored,
			  EXPAND_NO, FILL_NO, NO_PADDING);
      gtk_frame_set_label_align (GTK_FRAME (FrameXVTile), 0.5, 0);
	  gtk_container_add (GTK_CONTAINER (BoxXVTileH), BoxXVTileV1);
      gtk_container_add (GTK_CONTAINER (BoxXVTileH), BoxXVTileV2);
	  gtk_container_add (GTK_CONTAINER (FrameXVTile), BoxXVTileH);
      gtk_container_add (GTK_CONTAINER (BoxXVV), FrameXVTile);

      /*
       * Misc Frame 
       */
      gtk_box_pack_start (GTK_BOX (BoxXVMiscV),
			  wpMiscReverseVideoCheckButton, EXPAND_NO,
			  FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxXVMiscV),
			  wpMiscHorizontalFlipCheckButton,
			  EXPAND_NO, FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxXVMiscV),
			  wpMiscVerticalFlipCheckButton, EXPAND_NO,
			  FILL_NO, NO_PADDING);
      gtk_box_pack_start (GTK_BOX (BoxXVMiscV),
			  wpMiscSmoothCheckButton, EXPAND_NO, FILL_NO,
			  NO_PADDING);
      gtk_frame_set_label_align (GTK_FRAME (FrameXVMisc), 0.5, 0);
      gtk_container_add (GTK_CONTAINER (FrameXVMisc), BoxXVMiscV);
	  gtk_container_add (GTK_CONTAINER (BoxXVFillMiscH), FrameXVMisc);
    }

  gtk_container_add (GTK_CONTAINER (wpMainVBox), BoxTopH);

  /*
   * image setters 
   */
  gtk_container_add (GTK_CONTAINER (BoxMidV), BoxImageSettersH);
  gtk_container_add (GTK_CONTAINER (wpMainVBox), BoxMidV);
  /*
   * bottom separator 
   */
  gtk_box_pack_start_defaults (GTK_BOX (BoxMidV), wpSeparator);
  gtk_container_set_border_width (GTK_CONTAINER (BoxBotH), BORDER);
  gtk_container_add (GTK_CONTAINER (wpMainVBox), BoxBotH);

  gtk_container_add (GTK_CONTAINER (wpTopLevelWindow), wpMainVBox);

  /*--- Show widgets
  ------------------*/
  gtk_widget_show_all (wpTopLevelWindow);


  /*
   * Esetroot 
   */
  if (bHaveEsetroot)
    gtk_container_add (GTK_CONTAINER (BoxMidV), FrameEsetroot);
  /*
   * xv 
   */
  if (bHaveXV)
    gtk_container_add (GTK_CONTAINER (BoxMidV), BoxXVV);

  if (!strcmp(default_image_setter->str,"xv"))
	  if(bHaveXV)
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpXV), TRUE);
  else if (!strcmp(default_image_setter->str,"Esetroot"))
	  if(bHaveEsetroot)
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpEsetroot), TRUE);
  if (bHaveXV)
    {
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (wpXV)))
	gtk_widget_show_all (BoxXVV);
    }
  if (bHaveEsetroot)
    {
      if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (wpEsetroot)))
	gtk_widget_show_all (FrameEsetroot);
    }
}

/*initialize*/

int
bbrb_init (gint argc, gchar ** argv)
{
  gint i, j;
  gchar spCurrentStyle[1024];
  gchar spRootCommand[1024];
  GString *gsPATHS;
  GString *gsPATH;
  GString *gsPATHCOMMAND;

  /*
   * Initialize GTK+ 
   */
  gtk_init (&argc, &argv);

  /*
   * Handle command line arguments 
   */
  if (argc > 1)
    {
      for (j = 1; j < argc; j++)
	{
	  if ((!strcmp (argv[j], "--version"))
	      || (!strcmp (argv[j], "-version")))
	    {
	      printf ("%s %s\n", PACKAGE, VERSION);
	      printf ("Copyright (C) 1999-2001 Stacey Keast\n");
	      printf ("%s comes with ABSOLUTELY NO WARRANTY.\n", PACKAGE);
	      printf ("You may redistribute copies of %s", PACKAGE);
	      printf ("under the terms of the GNU General Public License.");
	      printf
		("For more information about these matters, see the file named COPYING.");
	    }
	  if ((!strcmp (argv[j], "--random"))
	      || (!strcmp (argv[j], "-random")))
	    {
	      bInit = TRUE;
	      if (PictureList_init ())
		return 1;
	      system (PictureList_getRandom ());
	      updatestyle_reset_no ("bbrb --random");
	      PictureList_destroy ();
	    }
	  else if (!strcmp (argv[j], "-preinst"))
	    {
	      if (resetBB ())
		{
		  printf
		    ("You don't appear to be running Blackbox 0.61.x or higher\n");
		  printf ("Run bbrb.inst again when running Blackbox.");
		  return -1;
		}
	    }
	  else if (!strcmp (argv[j], "-inst"))
	    {
	      GString *gsHome;
	      GString *gsBlackboxrc;
	      GString *gsMenu;
	      GString *gsMenuOld;
	      GString *gsBuffer;
	      GString *gsbgMenu;
	      gchar menuFile[1024], ch;
	      gint i = 0, haveStyles = 0;
	      FILE *fpin, *fpout;
	      if (resetBB ())
		return -1;

	      gsBuffer = g_string_new ("");
	      gsHome = g_string_new (getenv ("HOME"));
	      gsBlackboxrc = g_string_new (gsHome->str);
	      g_string_append (gsBlackboxrc, "/.blackboxrc");
	      gsMenu = g_string_new (gsHome->str);
	      g_string_append (gsMenu, "/.blackbox/Menu");
	      gsbgMenu = g_string_new (gsHome->str);
	      g_string_append (gsbgMenu, "/.blackbox/bgMenu");
	      copyfile (gsbgMenu->str, DEFAULTMENU);

	      /*
	       * if we get no error reading menuFile arg 
	       */
	      if (!getarg (gsBlackboxrc->str, "session.menuFile", menuFile))
		{
		  /*
		   * can we write to it? if (yes) 
		   */
		  if ((fpin = fopen (menuFile, "a")) != NULL)
		    {
		      fclose (fpin);
		      if (strcmp (gsMenu->str, menuFile))
			{
			  remove (gsMenu->str);
			  g_string_assign (gsMenu, menuFile);
			}
		    }
		}
	      gsMenuOld = g_string_new (gsMenu->str);
	      g_string_append (gsMenuOld, ".old");
	      setarg (gsBlackboxrc->str, "session.menuFile", gsMenu->str);

	      copyfile (gsMenuOld->str, gsMenu->str);
	      while (haveStyles < 4)
		{
		  fpin = fopen (gsMenuOld->str, "r");
		  fpout = fopen (gsMenu->str, "w");

		  while ((fgets (menuFile, 1024, fpin)) != NULL)
		    {
		      for (i = 0; i < strlen (menuFile); i++)
			{
			  ch = menuFile[i];
			  if (ch == '[')
			    {
			      g_string_assign (gsBuffer, "");
			      continue;
			    }
			  if (ch == ']')
			    {
			      if (!strcmp (gsBuffer->str, "stylesdir"))
				{
				  haveStyles = 1;
				}
			      if (!strcmp (gsBuffer->str, "end"))
				{
				  if (haveStyles == 1)
				    {
				      haveStyles = 2;
				    }
				}
			      if (!strcmp (gsBuffer->str, "begin")
				  && haveStyles == 3)
				haveStyles = 2;
			    }
			  g_string_append_c (gsBuffer, ch);
			}
		      fputs (menuFile, fpout);
		      if (haveStyles == 2)
			{
			  haveStyles = 4;
			  fprintf (fpout,
				   "\n  [include] (%s/.blackbox/bgMenu)\n",
				   gsHome->str);
			  printf
			    ("\nSuccessfully inserted [include] (%s/.blackbox/bgMenu) in \n%s",
			     gsHome->str, gsMenu->str);
			}
		    }
		  if (haveStyles == 0)
		    haveStyles = 3;
		  fclose (fpin);
		  fclose (fpout);
		}

	      resetBB ();

	      g_string_free (gsHome, TRUE);
	      g_string_free (gsBlackboxrc, TRUE);
	      g_string_free (gsMenu, TRUE);
	      g_string_free (gsMenuOld, TRUE);
	      g_string_free (gsBuffer, TRUE);
	      g_string_free (gsbgMenu, TRUE);
	    }
	  else
	    printf ("Usage: %s [--version] [--random]", argv[0]);
	}
      return 1;
    }

  /*
   * check and see what's available for image setters 
   */
  gsPATHS = g_string_new (getenv ("PATH"));
  gsPATH = g_string_new ("");
  gsPATHCOMMAND = g_string_new ("");
  bHaveXV = FALSE;
  bHaveEsetroot = FALSE;
  giNumberOfImageSetters = 0;
  for (i = 0; i < gsPATHS->len; i++)
    {
      if (gsPATHS->str[i] == ':')
	{
	  /*
	   * check for xv 
	   */
	  g_string_assign (gsPATHCOMMAND, gsPATH->str);
	  g_string_append (gsPATHCOMMAND, "/xv");
	  if (!access (gsPATHCOMMAND->str, X_OK)&&!bHaveXV)
	    {
	      bHaveXV = TRUE;
	      giNumberOfImageSetters++;
	    }
	  /*
	   * check for Esetroot 
	   */
	  g_string_assign (gsPATHCOMMAND, gsPATH->str);
	  g_string_append (gsPATHCOMMAND, "/Esetroot");
	  if (!access (gsPATHCOMMAND->str, X_OK)&&!bHaveEsetroot)
	    {
	      bHaveEsetroot = TRUE;
	      giNumberOfImageSetters++;
	    }
	  g_string_assign (gsPATH, "");
	  continue;
	}
      g_string_append_c (gsPATH, gsPATHS->str[i]);
    }
  g_string_free (gsPATHS, TRUE);
  g_string_free (gsPATH, TRUE);
  g_string_free (gsPATHCOMMAND, TRUE);
  if (!giNumberOfImageSetters)
    {
      printf
	("\nSorry, but you do not have xv or Esetroot in your $PATH or\n");
      printf
	("the path for these are on a NFS file system with UID mapping\n");
      printf
	("enabled.  You can try making a symlink to the executable from\n");
      printf ("another file system and add that to your path\n");
      exit (1);
    }

  /*
   * Something went wrong if we return 
   */
  bInit = TRUE;
  if (PictureList_init ())
    return 1;
  bInit = FALSE;

  /*
   * Initialize ImLib 
   */
  gdk_imlib_init ();
  gtk_widget_push_visual (gdk_imlib_get_visual ());
  gtk_widget_push_colormap (gdk_imlib_get_colormap ());

  ItemSelected = FALSE;
  ItemJustSelected = FALSE;

  getcurrentstyle (spCurrentStyle);
  getarg (spCurrentStyle, "rootCommand", spRootCommand);
  if (!strcmp (spRootCommand, "bbrb--random"))
    bRandomize = TRUE;
  else
    bRandomize = FALSE;
  /*
   * Create UI 
   */
  createUI ();
  bbrb_set_default_toggles();
  return 0;
}

/*Run*/

void
bbrb_run (void)
{
  /*
   * start GTK+ 
   */
  gtk_main ();
}

/*Entry point*/

gint
main (gint argc, gchar ** argv)
{
/*  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALEDIR);
  textdomain (PACKAGE); */
  if (!bbrb_init (argc, argv))
    {
      bbrb_run ();
      PictureList_destroy ();
    }

  return 0;
}

/*------------------
--Callbacks
--------------------*/

void
destroy_CB (GtkWidget * wpWidget, gpointer dpData)
{
  /*
   * Quit Program 
   */
  gtk_main_quit ();
}

gboolean
delete_event_CB (GtkWidget * wpWidget, GdkEvent * epEvent, gpointer dpData)
{
  /*
   * delete_event recieved from window manager 
   */
  return (FALSE);		/*do not prevent closure */
}

void
bbrb_set_toggles (void)
{
  /*
   * Esetroot 
   */
  if (bHaveEsetroot)
    {
      if (!strcmp (bbrb_fill->str, "1"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpEsetrootMax),
				      TRUE);
      else if (!strcmp (bbrb_fill->str, "2"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpEsetrootMaxAspect),
				      TRUE);
      else if (!strcmp (bbrb_tile->str, "5"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpEsetrootCentered),
				      TRUE);
      else
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpEsetrootTile),
				      TRUE);
    }

  /*
   * xv 
   */
  if (bHaveXV)
    {
      if (!strcmp (bbrb_fill->str, "0"))
	{
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpFillNo), TRUE);
	  gtk_widget_show (FrameXVTile);
	}
      else if (!strcmp (bbrb_fill->str, "1"))
	{
	  gtk_widget_hide (FrameXVTile);
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpFillMax), TRUE);
	  g_string_assign (bbrb_tile, "0");
	}
      else
	{
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpFillMaxAspect),
					TRUE);
	  gtk_widget_show (FrameXVTile);
	}
      if (!strcmp (bbrb_tile->str, "0"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpTile), TRUE);
      else if (!strcmp (bbrb_tile->str, "1"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpTileInteger),
				      TRUE);
      else if (!strcmp (bbrb_tile->str, "2"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpTileMirrored),
				      TRUE);
      else if (!strcmp (bbrb_tile->str, "3"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				      (wpTileIntegerMirrored), TRUE);
      else if (!strcmp (bbrb_tile->str, "4"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpTileCentered),
				      TRUE);
      else if (!strcmp (bbrb_tile->str, "5"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				      (wpTileCenteredSolidBackground), TRUE);
      else if (!strcmp (bbrb_tile->str, "8"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpTileSymmetrical),
				      TRUE);
      else if (!strcmp (bbrb_tile->str, "9"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				      (wpTileSymmetricalMirrored), TRUE);
      if (!strcmp (bbrb_reverse_video->str, "0"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				      (wpMiscReverseVideoCheckButton), FALSE);
      else
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				      (wpMiscReverseVideoCheckButton), TRUE);
      if (!strcmp (bbrb_flip_horizontal->str, "0"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				      (wpMiscHorizontalFlipCheckButton),
				      FALSE);
      else
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				      (wpMiscHorizontalFlipCheckButton),
				      TRUE);
      if (!strcmp (bbrb_flip_vertical->str, "0"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				      (wpMiscVerticalFlipCheckButton), FALSE);
      else
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				      (wpMiscVerticalFlipCheckButton), TRUE);
      if (!strcmp (bbrb_smooth->str, "0"))
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				      (wpMiscSmoothCheckButton), FALSE);
      else
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				      (wpMiscSmoothCheckButton), TRUE);
    }
}

void
bbrb_set_default_toggles (void)
{
	g_string_assign (bbrb_fill,default_fill->str);
	g_string_assign (bbrb_tile,default_tile->str);
	g_string_assign (bbrb_reverse_video,default_reverse_video->str);
	g_string_assign (bbrb_flip_horizontal,default_flip_horizontal->str);
	g_string_assign (bbrb_flip_vertical,default_flip_vertical->str);
	g_string_assign (bbrb_smooth,default_smooth->str);
	g_string_assign (bbrb_image_setter,default_image_setter->str);
	bbrb_set_toggles();
}

void
listitem_selected (GtkWidget * widget, gint row, gint column,
		   GdkEventButton * event, gpointer data)
{
  gtk_widget_set_sensitive (wpAddButton, FALSE);
  ItemSelected = TRUE;
  ItemJustSelected = TRUE;
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpRandomizeButton), FALSE);
  if (giRows > 1)
    gtk_widget_set_sensitive (wpRandomizeButton, TRUE);
  selected_row = row;
  PictureList_getRow (row);
  wpPixmap = bbrb_show_thumb (bbrb_file_name->str, wpPixmap, FALSE);
  if (!strcmp (bbrb_image_setter->str, "xv"))
	  if(bHaveXV)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpXV), TRUE);
  else if (!strcmp (bbrb_image_setter->str, "Esetroot"))
	  if(bHaveEsetroot)
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpEsetroot), TRUE);
  bbrb_set_toggles ();
  if (bHaveXV)
    gtk_widget_set_sensitive (BoxXVV, TRUE);
  if (bHaveEsetroot)
    gtk_widget_set_sensitive (FrameEsetroot, TRUE);

  gtk_widget_set_sensitive (wpRemoveButton, TRUE);
  gtk_widget_set_sensitive (wpRelabelButton, TRUE);
  gtk_widget_set_sensitive (wpSetRootButton, TRUE);
  ItemJustSelected = FALSE;
}

void
listitem_deselected (GtkWidget * widget, gint row, gint column,
		     GdkEventButton * event, gpointer data)
{
  selected_row = -1;
  gtk_widget_set_sensitive (wpAddButton, TRUE);
  ItemSelected = FALSE;
  bbrb_set_default_toggles();
  if(!strcmp(default_image_setter->str,"xv"))
	  if(bHaveXV)
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wpXV),TRUE);
  if(!strcmp(default_image_setter->str,"Esetroot"))
	  if(bHaveEsetroot)
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wpEsetroot),TRUE);
  gtk_widget_set_sensitive (wpRemoveButton, FALSE);
  gtk_widget_set_sensitive (wpRelabelButton, FALSE);
  gtk_widget_set_sensitive (wpSetRootButton, FALSE);
  wpPixmap = bbrb_show_thumb ("DEFAULTPIXMAP", wpPixmap, FALSE);
}

void
bbrb_update_item (GtkWidget * widget, gpointer gdata)
{
  if (!ItemJustSelected)
    {				/* doing this is redundant if an item has just been selected */
      gtk_clist_freeze (GTK_CLIST (wpPictureList));
		if(ItemSelected)
			gtk_widget_set_sensitive(wpUpdateButton,TRUE);
      if (bHaveEsetroot)
	{
	  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (wpEsetroot)))
	    {
	      if (GTK_TOGGLE_BUTTON (wpEsetrootTile)->active)
		{
		  if (selected_row < 0)
		    {
		      g_string_assign (default_tile, "0");
		      g_string_assign (default_fill, "0");
		    }
		  else
		    {
		      gtk_clist_set_text (GTK_CLIST (wpPictureList),
					  selected_row, 2, "0");
		      gtk_clist_set_text (GTK_CLIST (wpPictureList),
					  selected_row, 3, "0");
		    }
		}
	      else if (GTK_TOGGLE_BUTTON (wpEsetrootCentered)->active)
		{
		  if (selected_row < 0)
		    {
		      g_string_assign (default_fill, "0");
		      g_string_assign (default_tile, "5");
		    }
		  else
		    {
		      gtk_clist_set_text (GTK_CLIST (wpPictureList),
					  selected_row, 2, "0");
		      gtk_clist_set_text (GTK_CLIST (wpPictureList),
					  selected_row, 3, "5");
		    }
		}
	      else if (GTK_TOGGLE_BUTTON (wpEsetrootMax)->active)
		{
		  if (selected_row < 0)
		    g_string_assign (default_fill, "1");
		  else
		    {
		      gtk_clist_set_text (GTK_CLIST (wpPictureList),
					  selected_row, 2, "1");
		    }
		  if (bHaveXV)
		    gtk_widget_hide (FrameXVTile);
		}
	      else
		{
		  if (selected_row < 0)
		    {
		      g_string_assign (default_fill, "2");
		      g_string_assign (default_tile, "5");
		    }
		  else
		    {
		      gtk_clist_set_text (GTK_CLIST (wpPictureList),
					  selected_row, 2, "2");
		      gtk_clist_set_text (GTK_CLIST (wpPictureList),
					  selected_row, 3, "5");
		    }
		}
	    }
	}

      /*
       * xv 
       */
      if (bHaveXV)
	{
	  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (wpXV)))
	    {
	      if (GTK_TOGGLE_BUTTON (wpTile)->active)
		{
		  if (selected_row < 0)
		    g_string_assign (default_tile, "0");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 3, "0");
		}
	      else if (GTK_TOGGLE_BUTTON (wpTileInteger)->active)
		{
		  if (selected_row < 0)
		    g_string_assign (default_tile, "1");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 3, "1");
		}
	      else if (GTK_TOGGLE_BUTTON (wpTileMirrored)->active)
		{
		  if (selected_row < 0)
		    g_string_assign (default_tile, "2");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 3, "2");
		}
	      else if (GTK_TOGGLE_BUTTON (wpTileIntegerMirrored)->active)
		{
		  if (selected_row < 0)
		    g_string_assign (default_tile, "3");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 3, "3");
		}
	      else if (GTK_TOGGLE_BUTTON (wpTileCentered)->active)
		{
		  if (selected_row < 0)
		    g_string_assign (default_tile, "4");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 3, "4");
		}
	      else if (GTK_TOGGLE_BUTTON (wpTileCenteredSolidBackground)->
		       active)
		{
		  if (selected_row < 0)
		    g_string_assign (default_tile, "5");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 3, "5");
		}
	      else if (GTK_TOGGLE_BUTTON (wpTileSymmetrical)->active)
		{
		  if (selected_row < 0)
		    g_string_assign (default_tile, "8");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 3, "8");
		}
	      else
		{
		  if (selected_row < 0)
		    g_string_assign (default_tile, "9");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 3, "9");
		}

	      if (GTK_TOGGLE_BUTTON (wpFillNo)->active)
		{
		  gtk_widget_show (FrameXVTile);
		  if (selected_row < 0)
		    g_string_assign (default_fill, "0");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 2, "0");
		}
	      else if (GTK_TOGGLE_BUTTON (wpFillMax)->active)
		{
		  gtk_widget_hide (FrameXVTile);
		  if (selected_row < 0)
		    g_string_assign (default_fill, "1");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 2, "1");
		}
	      else
		{
		  if (selected_row < 0)
		    g_string_assign (default_fill, "2");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 2, "2");
		  gtk_widget_show (FrameXVTile);
		}

	      if (GTK_TOGGLE_BUTTON (wpMiscReverseVideoCheckButton)->active)
		{
		  if (selected_row < 0)
		    g_string_assign (default_reverse_video, "1");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 4, "1");
		}
	      else
		{
		  if (selected_row < 0)
		    g_string_assign (default_reverse_video, "0");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 4, "0");
		}
	      if (GTK_TOGGLE_BUTTON (wpMiscHorizontalFlipCheckButton)->active)
		{
		  if (selected_row < 0)
		    g_string_assign (default_flip_horizontal, "1");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 5, "1");
		}
	      else
		{
		  if (selected_row < 0)
		    g_string_assign (default_flip_horizontal, "0");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 5, "0");
		}
	      if (GTK_TOGGLE_BUTTON (wpMiscVerticalFlipCheckButton)->active)
		{
		  if (selected_row < 0)
		    g_string_assign (default_flip_vertical, "1");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 6, "1");
		}
	      else
		{
		  if (selected_row < 0)
		    g_string_assign (default_flip_vertical, "0");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 6, "0");
		}
	      if (GTK_TOGGLE_BUTTON (wpMiscSmoothCheckButton)->active)
		{
		  if (selected_row < 0)
		    g_string_assign (default_smooth, "1");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 7, "1");
		}
	      else
		{
		  if (selected_row < 0)
		    g_string_assign (default_smooth, "0");
		  else
		    gtk_clist_set_text (GTK_CLIST (wpPictureList),
					selected_row, 7, "0");
		}
	    }
	}
      gtk_clist_thaw (GTK_CLIST (wpPictureList));
    }
}

void
bbrb_list_remove (GtkWidget * widget, gpointer gdata)
{
  gtk_clist_remove (GTK_CLIST (wpPictureList), selected_row);
  giRows--;
  if (giRows < 2)
    {
      gtk_widget_set_sensitive (wpSortButton, FALSE);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpRandomizeButton),
				    FALSE);
      gtk_widget_set_sensitive (wpRandomizeButton, FALSE);
      bRandomize = FALSE;
    }
  else
    gtk_widget_set_sensitive (wpSortButton, TRUE);
  if (giRows < 1)
    gtk_widget_set_sensitive (wpClearButton, FALSE);
  gtk_widget_set_sensitive (wpUpdateButton, TRUE);
  ItemSelected = FALSE;
}

void
bbrb_list_sort (GtkWidget * widget, gpointer gdata)
{
  gtk_clist_sort (GTK_CLIST (wpPictureList));
  gtk_widget_set_sensitive (wpUpdateButton, TRUE);
  gtk_clist_unselect_all (GTK_CLIST (wpPictureList));
  ItemSelected = FALSE;
}

void
bbrb_set_root (GtkWidget * widget, gpointer gdata)
{
  PictureList_getRootCommand (selected_row);
  if (bRandomize)
    {
      system (PictureList_getRandom ());
      updatestyle_reset_no ("bbrb --random");
    }
  else
    updatestyle (bbrb_root_command->str);
}

void
bbrb_set_label (GtkWidget * widget, gpointer gdata)
{
  PictureList_setLabel (selected_row);
  gtk_widget_set_sensitive (wpUpdateButton, TRUE);
}

void
bbrb_update (GtkWidget * widget, gpointer gdata)
{
  GString *gsbbrbrootcommand;
  if (bHaveXV)
    gsbbrbrootcommand = g_string_new ("xv -root -quit -maxpect -rmode 5 ");
  else if (bHaveEsetroot)
    gsbbrbrootcommand = g_string_new ("Esetroot ");
  g_string_append (gsbbrbrootcommand, DEFAULTPIXMAP);
  PictureList_update ();
  if (giRows == 1)
    updatestyle_reset_no (PictureList_getRootCommand (0));
  /*
   * FIXME maybe have this go back to the default background for the style used 
   */
  if (giRows == 0)
    updatestyle_reset_no (gsbbrbrootcommand->str);
  g_string_free (gsbbrbrootcommand, TRUE);
}

void
bbrb_list_add (GtkWidget * widget, gpointer gdata)
{
  PictureList_add ();
}

void
bbrb_list_clear (GtkWidget * widget, gpointer gdata)
{
  gtk_clist_clear (GTK_CLIST (wpPictureList));
  gtk_widget_set_sensitive (wpUpdateButton, TRUE);
  gtk_widget_set_sensitive (wpClearButton, FALSE);
  gtk_widget_set_sensitive (wpRelabelButton, FALSE);
  gtk_widget_set_sensitive (wpRemoveButton, FALSE);
  gtk_widget_set_sensitive (wpSortButton, FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wpRandomizeButton), FALSE);
  gtk_widget_set_sensitive (wpRandomizeButton, FALSE);
  gtk_widget_set_sensitive (wpSetRootButton, FALSE);
  giRows = 0;
  selected_row = -1;
  ItemSelected = FALSE;
  gtk_widget_set_sensitive (wpAddButton, TRUE);
}

void
bbrb_randomize_toggled (GtkWidget * widget, gpointer gdata)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (wpRandomizeButton)))
    {
      gtk_clist_unselect_all (GTK_CLIST (wpPictureList));
      bRandomize = TRUE;
      ItemSelected = FALSE;
    }
  else
    bRandomize = FALSE;
  gtk_widget_set_sensitive (wpSetRootButton, TRUE);
  gtk_widget_set_sensitive (wpRandomizeButton, FALSE);
}

void
bbrb_show_xv (GtkWidget * widget, gpointer gdata)
{
  if (bHaveEsetroot)
    gtk_widget_hide (FrameEsetroot);
  gtk_widget_show_all (BoxXVV);
  if (ItemSelected)
    {
      PictureList_getRow (selected_row);
      bbrb_set_toggles ();
      gtk_clist_set_text (GTK_CLIST (wpPictureList), selected_row, 1, "xv");
		if(!ItemJustSelected)
      gtk_widget_set_sensitive (wpUpdateButton, TRUE);
    }
	else
	{
		g_string_assign(default_image_setter,"xv");
		bbrb_set_default_toggles();
	}
}

void
bbrb_show_Esetroot (GtkWidget * widget, gpointer gdata)
{
  if (bHaveXV)
    gtk_widget_hide (BoxXVV);
  gtk_widget_show_all (FrameEsetroot);
  if (ItemSelected)
    {
      PictureList_getRow (selected_row);
      bbrb_set_toggles ();
      gtk_clist_set_text (GTK_CLIST (wpPictureList), selected_row, 1,
			  "Esetroot");
		if(!ItemJustSelected)
      gtk_widget_set_sensitive (wpUpdateButton, TRUE);
    }
	else
	{
		g_string_assign(default_image_setter,"Esetroot");
		bbrb_set_default_toggles();
	}
}
