/* Copyright (C) 1999-2002 Stacey L. Keast <slik@telusplanet.net>
  
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software 
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "bbrblist.h"

void
destroy_dialog_CB (GtkWidget * widget, gpointer * data)
{
  gtk_grab_remove (widget);
}

void
dialog_relabel_ok (GtkWidget * widget, gpointer gdata)
{
  gtk_clist_set_text (GTK_CLIST (wpPictureList), selected_row, 0,
		      gtk_entry_get_text (GTK_ENTRY (wpEntryBox)));
  gtk_widget_destroy (GTK_WIDGET (wpEntryBox));
  gtk_widget_destroy (GTK_WIDGET (gdata));
}

void
PictureList_setLabel (gint row)
{
  GtkWidget *wpRelabelDialog;
  GtkWidget *wpRelabelOKButton;
  GString *gsLabelName;
  selected_row = row;
  wpRelabelDialog = gtk_dialog_new ();
  gtk_signal_connect (GTK_OBJECT (wpRelabelDialog), "destroy",
		      GTK_SIGNAL_FUNC (destroy_dialog_CB), NULL);
  gtk_window_set_title (GTK_WINDOW (wpRelabelDialog), "Enter a new label...");
  gtk_container_border_width (GTK_CONTAINER (wpRelabelDialog), 3);
  wpEntryBox = gtk_entry_new_with_max_length (50);
  gtk_signal_connect (GTK_OBJECT (wpEntryBox), "activate",
		      GTK_SIGNAL_FUNC (dialog_relabel_ok), wpRelabelDialog);
  PictureList_getRow (row);
  gtk_entry_set_text (GTK_ENTRY (wpEntryBox), bbrb_label->str);
  gtk_entry_set_editable (GTK_ENTRY (wpEntryBox), TRUE);
  gtk_entry_select_region (GTK_ENTRY (wpEntryBox), 0,
			   GTK_ENTRY (wpEntryBox)->text_length);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (wpRelabelDialog)->vbox),
		      wpEntryBox, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (wpEntryBox, GTK_CAN_DEFAULT);
  wpRelabelOKButton = gtk_button_new_with_label ("OK");
  gtk_signal_connect (GTK_OBJECT (wpRelabelOKButton), "clicked",
		      GTK_SIGNAL_FUNC (dialog_relabel_ok), wpRelabelDialog);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (wpRelabelDialog)->action_area),
		      wpRelabelOKButton, TRUE, TRUE, 0);
  gtk_grab_add (wpRelabelDialog);
  gtk_widget_grab_focus (wpEntryBox);
  gtk_widget_show_all (wpRelabelDialog);
}

void
PictureList_destroy (void)
{
  FILE *fpBuffer;
  GString *gsHomeDir;
  GString *gsDataFile;
  gsHomeDir = g_string_new (getenv ("HOME"));
  gsDataFile = g_string_new (gsHomeDir->str);
  g_string_append (gsDataFile, "/.bbrbrc");
  setarg(gsDataFile->str,"ImageSetter",default_image_setter->str);
  setarg(gsDataFile->str,"Tile",default_tile->str);
  setarg(gsDataFile->str,"Fill",default_fill->str);
  setarg(gsDataFile->str,"ReverseVideo",default_reverse_video->str);
  setarg(gsDataFile->str,"FlipHorizontal",default_flip_horizontal->str);
  setarg(gsDataFile->str,"FlipVertical",default_flip_vertical->str);
  setarg(gsDataFile->str,"Smooth",default_smooth->str);
  g_string_free (gsHomeDir, TRUE);
  g_string_free (gsDataFile, TRUE);
  g_string_free (bbrb_label, TRUE);
  g_string_free (bbrb_image_setter, TRUE);
  g_string_free (default_image_setter, TRUE);
  g_string_free (bbrb_file_name, TRUE);
  g_string_free (bbrb_root_command, TRUE);
  g_string_free (bbrb_fill, TRUE);
  g_string_free (default_fill, TRUE);
  g_string_free (bbrb_tile, TRUE);
  g_string_free (default_tile, TRUE);
  g_string_free (bbrb_reverse_video, TRUE);
  g_string_free (default_reverse_video, TRUE);
  g_string_free (bbrb_flip_horizontal, TRUE);
  g_string_free (default_flip_horizontal, TRUE);
  g_string_free (bbrb_flip_vertical, TRUE);
  g_string_free (default_flip_vertical, TRUE);
  g_string_free (bbrb_smooth, TRUE);
  g_string_free (default_smooth, TRUE);
  g_string_free (gsRandomPicStripped, TRUE);
}

int
PictureList_init (void)
{
  FILE *fpBuffer;
  GString *gsHomeDir;
  GString *gsDataFile;
  char arg[1024];
  gsHomeDir = g_string_new (getenv ("HOME"));
  gsDataFile = g_string_new (gsHomeDir->str);
  g_string_append (gsDataFile, "/.blackbox/bgMenu");
  if ((fpBuffer = fopen (gsDataFile->str, "r")) == NULL)
    {
      copyfile (gsDataFile->str, DEFAULTMENU);
      printf ("\nPlease run bbrb.inst first.\n");
      g_string_free (gsHomeDir, TRUE);
      g_string_free (gsDataFile, TRUE);
      return 1;
    }
  else
    fclose (fpBuffer);
  
  bbrb_label = g_string_new ("");

  g_string_assign(gsDataFile,gsHomeDir->str);
  g_string_append(gsDataFile,"/.bbrbrc");
  if ((fpBuffer=fopen(gsDataFile->str,"a"))!=NULL)
	  fclose(fpBuffer);
  if(getarg (gsDataFile->str,"ImageSetter",arg))
  {
	  if(bHaveXV)
		  default_image_setter=g_string_new("xv");
	  else if (bHaveEsetroot)
		  default_image_setter=g_string_new("Esetroot");
	  else
	  {
		  printf("no xv of Esetroot found.\n");
		  exit (EXIT_FAILURE);
	  }
	  setarg(gsDataFile->str,"ImageSetter",default_image_setter->str);
  }
  else
	  default_image_setter=g_string_new(arg);
  bbrb_image_setter = g_string_new("");
  
  if(getarg (gsDataFile->str,"Fill",arg))
  {
	  default_fill=g_string_new("0");
	  setarg(gsDataFile->str,"Fill",default_fill->str);
  }
  else
	  default_fill=g_string_new(arg);
  bbrb_fill = g_string_new("");
  
  if(getarg (gsDataFile->str,"Tile",arg))
  {
	  default_tile=g_string_new("0");
	  setarg(gsDataFile->str,"Tile",default_tile->str);
  }
  else
	  default_tile=g_string_new(arg);
  bbrb_tile = g_string_new("");
  
  if(getarg (gsDataFile->str,"ReverseVideo",arg))
  {
	  default_reverse_video=g_string_new("0");
	  setarg(gsDataFile->str,"ReverseVideo",default_reverse_video->str);
  }
  else
	  default_reverse_video=g_string_new(arg);
  bbrb_reverse_video = g_string_new("");
  
  if(getarg (gsDataFile->str,"FlipHorizontal",arg))
  {
	  default_flip_horizontal=g_string_new("0");
	  setarg(gsDataFile->str,"FlipHorizontal",default_flip_horizontal->str);
  }
  else
	  default_flip_horizontal=g_string_new(arg);
  bbrb_flip_horizontal = g_string_new("");
  
  if(getarg (gsDataFile->str,"FlipVertical",arg))
  {
	  default_flip_vertical=g_string_new("0");
	  setarg(gsDataFile->str,"FlipVertical",default_flip_vertical->str);
  }
  else
	  default_flip_vertical=g_string_new(arg);
  bbrb_flip_vertical = g_string_new("");
  
  if(getarg (gsDataFile->str,"Smooth",arg))
  {
	  default_smooth=g_string_new("0");
	  setarg(gsDataFile->str,"Smooth",default_smooth->str);
  }
  else
	  default_smooth=g_string_new(arg);
  bbrb_smooth = g_string_new("");

  g_string_free (gsHomeDir, TRUE);
  g_string_free (gsDataFile, TRUE);
  
  bbrb_file_name = g_string_new ("");
  bbrb_root_command = g_string_new ("");
  gsRandomPicStripped = g_string_new ("");
  wpPictureList = gtk_clist_new (9);
  gtk_clist_set_selection_mode (GTK_CLIST (wpPictureList),
				GTK_SELECTION_SINGLE);
  gtk_clist_set_shadow_type (GTK_CLIST (wpPictureList), GTK_SHADOW_NONE);
  gtk_clist_column_titles_hide (GTK_CLIST (wpPictureList));
  gtk_clist_set_column_width (GTK_CLIST (wpPictureList), 0, 300);
  gtk_clist_set_sort_column (GTK_CLIST (wpPictureList), 0);
  gtk_clist_set_sort_type (GTK_CLIST (wpPictureList), GTK_SORT_ASCENDING);
  selected_row=-1;
  giRows = 0;
  bbrb_load_list ();
  g_string_assign (bbrb_image_setter,default_image_setter->str);
  g_string_assign (bbrb_fill, default_fill->str);
  g_string_assign (bbrb_tile, default_tile->str);
  g_string_assign (bbrb_reverse_video, default_reverse_video->str);
  g_string_assign (bbrb_flip_horizontal, default_flip_horizontal->str);
  g_string_assign (bbrb_flip_vertical, default_flip_vertical->str);
  g_string_assign (bbrb_smooth, default_smooth->str);
  return 0;
}

void
bbrb_load_list (void)
{
  FILE *fpBuffer;
  gchar spBuffer[1024];
  GString *gsBuffer;
  gchar ch;
  GString *gsHomeDir;
  GString *gsDataFile;
  gboolean AddItem;
  gboolean HaveImageSetter;
  gboolean HaveRMode;
  gboolean HaveQuotes;
  gint i;

  gsHomeDir = g_string_new (getenv ("HOME"));
  gsDataFile = g_string_new (gsHomeDir->str);
  g_string_append (gsDataFile, "/.blackbox/bgMenu");
  gtk_clist_freeze (GTK_CLIST (wpPictureList));

  fpBuffer = fopen (gsDataFile->str, "r");
  gsBuffer = g_string_new ("");
  while ((fgets (spBuffer, 1024, fpBuffer)) != NULL)
    {
      AddItem = TRUE;
      HaveQuotes = FALSE;
      for (i = 0; i < strlen (spBuffer); i++)
	{
	  ch = spBuffer[i];
	  if (ch == '\"')
	    {
	      if (HaveQuotes)
		HaveQuotes = FALSE;
	      else
		HaveQuotes = TRUE;
	    }
	  if (ch == '[' || ch == '(' || ch == '{')
	    {
	      g_string_assign (gsBuffer, "");
	      continue;
	    }
	  if (ch == ']')
	    {
	      if (!strcmp (gsBuffer->str, "submenu"))
		{
		  AddItem = FALSE;
		  g_string_assign (gsBuffer, "");
		  break;
		}
	      if (!strcmp (gsBuffer->str, "end"))
		{
		  AddItem = FALSE;
		  g_string_assign (gsBuffer, "");
		  break;
		}
	    }
	  if (ch == ')')
	    {
	      if (!strcmp (gsBuffer->str, "Edit List")
		  || !strcmp (gsBuffer->str, "Random"))
		{
		  AddItem = FALSE;
		  g_string_assign (gsBuffer, "");
		  break;
		}
	      g_string_assign (bbrb_label, gsBuffer->str);
	      continue;
	    }
	  if (ch == '}' && !HaveQuotes)
	    {
	      g_string_assign (bbrb_root_command, gsBuffer->str);
	      g_string_assign (gsBuffer, "");
	      break;
	    }
	  g_string_append_c (gsBuffer, ch);
	}
      if (AddItem)
	{
	  HaveImageSetter = FALSE;
	  HaveRMode = FALSE;
	  HaveQuotes = FALSE;
	  g_string_assign (gsBuffer, "");
	  g_string_assign (bbrb_fill, "0");
	  g_string_assign (bbrb_tile, "0");
	  g_string_assign (bbrb_reverse_video, "0");
	  g_string_assign (bbrb_flip_horizontal, "0");
	  g_string_assign (bbrb_flip_vertical, "0");
	  g_string_assign (bbrb_smooth, "0");
	  for (i = 0; i < bbrb_root_command->len; i++)
	    {
	      ch = bbrb_root_command->str[i];
	      if (ch == '\"')
		{
		  if (HaveQuotes)
		    HaveQuotes = FALSE;
		  else
		    HaveQuotes = TRUE;
		}
	      if (ch == ' ' && !HaveQuotes)
		{
		  if (HaveImageSetter)
		    {
		      HaveImageSetter = FALSE;
		      g_string_assign (bbrb_image_setter, gsBuffer->str);
		      g_string_assign (gsBuffer, "");
		      continue;
		    }
		  /* xv */
		  if (HaveRMode)
		    {
		      HaveRMode = FALSE;
		      g_string_assign (bbrb_tile, gsBuffer->str);
		      g_string_assign (gsBuffer, "");
		      continue;
		    }
		  if ((!strcmp (gsBuffer->str, "-root"))
		      || (!strcmp (gsBuffer->str, "-quit")))
		    {
		      g_string_assign (gsBuffer, "");
		      continue;
		    }
		  if (!strcmp (gsBuffer->str, "-rmode"))
		    {
		      HaveRMode = TRUE;
		      g_string_assign (gsBuffer, "");
		      continue;
		    }
		  if (!strcmp (gsBuffer->str, "-rv"))
		    {
		      g_string_assign (bbrb_reverse_video, "1");
		      g_string_assign (gsBuffer, "");
		      continue;
		    }
		  if (!strcmp (gsBuffer->str, "-max"))
		    {
		      g_string_assign (bbrb_fill, "1");
		      g_string_assign (gsBuffer, "");
		      continue;
		    }
		  if (!strcmp (gsBuffer->str, "-maxpect"))
		    {
		      g_string_assign (bbrb_fill, "2");
		      g_string_assign (gsBuffer, "");
		      continue;
		    }
		  if (!strcmp (gsBuffer->str, "-hflip"))
		    {
		      g_string_assign (bbrb_flip_horizontal, "1");
		      g_string_assign (gsBuffer, "");
		      continue;
		    }
		  if (!strcmp (gsBuffer->str, "-vflip"))
		    {
		      g_string_assign (bbrb_flip_vertical, "1");
		      g_string_assign (gsBuffer, "");
		      continue;
		    }
		  if (!strcmp (gsBuffer->str, "-smooth"))
		    {
		      g_string_assign (bbrb_smooth, "1");
		      g_string_assign (gsBuffer, "");
		      continue;
		    }
		  /* Esetroot */
		  if (!strcmp (gsBuffer->str, "-s"))
		    {
		      g_string_assign (bbrb_fill, "1");
		      g_string_assign (gsBuffer, "");
		      continue;
		    }
		  if (!strcmp (gsBuffer->str, "-c"))
		    {
		      g_string_assign (bbrb_tile, "5");
		      g_string_assign (gsBuffer, "");
		      continue;
		    }
		  if (!strcmp (gsBuffer->str, "-f"))
		    {
		      g_string_assign (bbrb_tile, "5");
		      g_string_assign (bbrb_fill, "2");
		      g_string_assign (gsBuffer, "");
		      continue;
		    }
		}
	      if (ch == '\'')
		{
		  HaveImageSetter = TRUE;
		  if (!strcmp (gsBuffer->str, "bbrbss "))
		    g_string_assign (gsBuffer, "");
		}
	      else
		g_string_append_c (gsBuffer, ch);
	    }
	  g_string_assign (bbrb_file_name, "");
	  for (i = 0; i < gsBuffer->len; i++)
	    {
	      ch = gsBuffer->str[i];
	      if (ch != '\\' && ch != '\"')
		g_string_append_c (bbrb_file_name, ch);
	    }
	  if (!bHaveXV && !strcmp (bbrb_image_setter->str, "xv"))
	    {
	      if (bHaveEsetroot)
		g_string_assign (bbrb_image_setter, "Esetroot");
	      /* else if others */
	    }
	  if (!bHaveEsetroot && !strcmp (bbrb_image_setter->str, "Esetroot"))
	    {
	      if (bHaveXV)
		g_string_assign (bbrb_image_setter, "xv");
	      /* else if others */
	    }
	  PictureList_addRow ();
	}
    }
  g_string_free (gsBuffer, TRUE);
  g_string_free (gsHomeDir, TRUE);
  g_string_free (gsDataFile, TRUE);
  fclose (fpBuffer);
  gtk_clist_thaw (GTK_CLIST (wpPictureList));
}

void
PictureList_addRow (void)
{
  gchar *clItem[9] = { bbrb_label->str,
    bbrb_image_setter->str,
    bbrb_fill->str,
    bbrb_tile->str,
    bbrb_reverse_video->str,
    bbrb_flip_horizontal->str,
    bbrb_flip_vertical->str,
    bbrb_smooth->str,
    bbrb_file_name->str
  };
  gtk_clist_append (GTK_CLIST (wpPictureList), clItem);
  if (!bInit)
    gtk_widget_set_sensitive (wpClearButton, TRUE);
  giRows++;
  if (giRows > 1 && !bInit)
    gtk_widget_set_sensitive (GTK_WIDGET (wpRandomizeButton), TRUE);
}

gchar *
PictureList_getRootCommand (gint row)
{
  gint i;
  GString *gsRootFileName = g_string_new ("");
  PictureList_getRow (row);
  for (i = 0; i < bbrb_file_name->len; i++)
    {
      if (bbrb_file_name->str[i] == '(' || bbrb_file_name->str[i] == '&'
	  || bbrb_file_name->str[i] == ' ')
	g_string_append_c (gsRootFileName, '\\');
      g_string_append_c (gsRootFileName, bbrb_file_name->str[i]);
    }
  g_string_assign (bbrb_root_command, bbrb_image_setter->str);
  g_string_append_c (bbrb_root_command, ' ');
  /* Esetroot */
  if (!strcmp (bbrb_image_setter->str, "Esetroot"))
    {
      if (!strcmp (bbrb_fill->str, "1"))
	g_string_append (bbrb_root_command, "-s ");
      else if (!strcmp (bbrb_fill->str, "2"))
	g_string_append (bbrb_root_command, "-f ");
      else if (!strcmp (bbrb_tile->str, "5"))
	g_string_append (bbrb_root_command, "-c ");
    }
  /* xv */
  if (!strcmp (bbrb_image_setter->str, "xv"))
    {
      g_string_append (bbrb_root_command, "-root -quit ");
      if (!strcmp (bbrb_fill->str, "1"))
	g_string_append (bbrb_root_command, "-max ");
      else if (!strcmp (bbrb_fill->str, "2"))
	{
	  g_string_append (bbrb_root_command, "-maxpect -rmode ");
	  g_string_append (bbrb_root_command, bbrb_tile->str);
	  g_string_append_c (bbrb_root_command, ' ');
	}
      else
	{
	  g_string_append (bbrb_root_command, "-rmode ");
	  g_string_append (bbrb_root_command, bbrb_tile->str);
	  g_string_append_c (bbrb_root_command, ' ');
	}
      if (!strcmp (bbrb_reverse_video->str, "1"))
	g_string_append (bbrb_root_command, "-rv ");
      if (!strcmp (bbrb_flip_horizontal->str, "1"))
	g_string_append (bbrb_root_command, "-hflip ");
      if (!strcmp (bbrb_flip_vertical->str, "1"))
	g_string_append (bbrb_root_command, "-vflip ");
      if (!strcmp (bbrb_smooth->str, "1"))
	g_string_append (bbrb_root_command, "-smooth ");
    }
  g_string_append_c (bbrb_root_command, '\"');
  g_string_append (bbrb_root_command, gsRootFileName->str);
  g_string_append_c (bbrb_root_command, '\"');
  g_string_free (gsRootFileName, TRUE);
  return bbrb_root_command->str;
}

void
PictureList_getRow (gint row)
{
  gchar *spTemp;
  gtk_clist_get_text (GTK_CLIST (wpPictureList), row, 0, &spTemp);
  g_string_assign (bbrb_label, spTemp);
  gtk_clist_get_text (GTK_CLIST (wpPictureList), row, 1, &spTemp);
  g_string_assign (bbrb_image_setter, spTemp);
  gtk_clist_get_text (GTK_CLIST (wpPictureList), row, 2, &spTemp);
  g_string_assign (bbrb_fill, spTemp);
  gtk_clist_get_text (GTK_CLIST (wpPictureList), row, 3, &spTemp);
  g_string_assign (bbrb_tile, spTemp);
  gtk_clist_get_text (GTK_CLIST (wpPictureList), row, 4, &spTemp);
  g_string_assign (bbrb_reverse_video, spTemp);
  gtk_clist_get_text (GTK_CLIST (wpPictureList), row, 5, &spTemp);
  g_string_assign (bbrb_flip_horizontal, spTemp);
  gtk_clist_get_text (GTK_CLIST (wpPictureList), row, 6, &spTemp);
  g_string_assign (bbrb_flip_vertical, spTemp);
  gtk_clist_get_text (GTK_CLIST (wpPictureList), row, 7, &spTemp);
  g_string_assign (bbrb_smooth, spTemp);
  gtk_clist_get_text (GTK_CLIST (wpPictureList), row, 8, &spTemp);
  g_string_assign (bbrb_file_name, spTemp);
}

void
PictureList_update (void)
{
  FILE *fpDataFile;
  GString *gsDataFile;
  gint i;
  gsDataFile = g_string_new (getenv ("HOME"));
  g_string_append (gsDataFile, "/.blackbox/bgMenu");
  if ((fpDataFile = fopen (gsDataFile->str, "w")) != NULL)
    {
      fprintf (fpDataFile,
	       "[submenu]   (Backgrounds)      {Choose a background...}\n");
      fprintf (fpDataFile, "  [exec]    (Edit List)        {bbrb}\n");
      if (giRows > 1)
	fprintf (fpDataFile,
		 "  [exec]    (Random)           {bbrb --random}\n");
      for (i = 0; i < giRows; i++)
	{
	  PictureList_getRow (i);
	  fprintf (fpDataFile, "  [exec]    (%s)  {bbrbss \'%s\'}\n",
		   bbrb_label->str, PictureList_getRootCommand (i));
	}
      fprintf (fpDataFile, "[end]\n");
      fclose (fpDataFile);
    }
  g_string_free (gsDataFile, TRUE);
}

void
filecancel (GtkWidget * widget, gpointer gdata)
{
  gtk_grab_remove (wpFileThumb);
  gtk_widget_destroy (wpFileThumb);
}

void
bbrb_add_filename (gchar * filename)
{
  GString *gsLabelBuffer = g_string_new ("");
  if (filename[strlen (filename) - 1] != '/')
    {
      extractname (filename, gsLabelBuffer);
      g_string_assign (bbrb_label, gsLabelBuffer->str);
      g_string_assign (bbrb_file_name, filename);
      g_string_assign (bbrb_fill, default_fill->str);
      g_string_assign (bbrb_tile, default_tile->str);
      g_string_assign (bbrb_image_setter, default_image_setter->str);
      g_string_assign (bbrb_reverse_video, default_reverse_video->str);
      g_string_assign (bbrb_flip_horizontal, default_flip_horizontal->str);
      g_string_assign (bbrb_flip_vertical, default_flip_vertical->str);
      g_string_assign (bbrb_smooth, default_smooth->str);
      /*g_string_assign (bbrb_root_command, "0");*/
      PictureList_addRow ();
      gtk_widget_set_sensitive (wpUpdateButton, TRUE);
      if (giRows < 2)
	gtk_widget_set_sensitive (wpSortButton, FALSE);
      else
	gtk_widget_set_sensitive (wpSortButton, TRUE);
      gtk_clist_unselect_all (GTK_CLIST (wpPictureList));
    }
  g_string_free (gsLabelBuffer, TRUE);
}

void
file_ok_sel (GtkWidget * w, GtkFileSelection * fs)
{
  FILE *fpTemp;
  GString *gsBuffer,*gsPath,*gsbbrbrc;
  gsPath = g_string_new ("");
  gsbbrbrc = g_string_new (getenv("HOME"));
  g_string_append(gsbbrbrc,"/.bbrbrc");
  gsBuffer = g_string_new (file_thumb_get_filename (FILE_THUMB (fs)));
  extractpath(gsBuffer->str,gsPath);
  setarg(gsbbrbrc->str,"Path",gsPath->str);
  g_string_free(gsPath,TRUE);
  g_string_free(gsbbrbrc,TRUE);
  if ((fpTemp = fopen (gsBuffer->str, "r")) != NULL)
    {
      fclose (fpTemp);
      bbrb_add_filename (gsBuffer->str);
    }
  g_string_free (gsBuffer, TRUE);
  filecancel (NULL, NULL);
}

void
PictureList_add_loadall (GtkWidget * widget, gpointer gdata)
{
  gint i;
  gint ii = 0;
  gchar *fileitem;
  gchar *current_path;
  GString *gsBuffer;
  GString *gsBufferCurrentPath;

  gsBuffer = g_string_new ("");
  gsBufferCurrentPath = g_string_new ("");
  gtk_label_get (GTK_LABEL (FILE_THUMB (wpFileThumb)->selection_text),
		 &current_path);
  for (ii = 11; ii < strlen (current_path); ii++)
    g_string_append_c (gsBufferCurrentPath, current_path[ii]);
  for (i = 0; i <= FILE_THUMB (wpFileThumb)->last_file_row; i++)
    {
      if (gtk_clist_get_text
	  (GTK_CLIST (FILE_THUMB (wpFileThumb)->file_list), i, 0, &fileitem))
	{
	  g_string_assign (gsBuffer, gsBufferCurrentPath->str);
	  g_string_append_c (gsBuffer, '/');
	  g_string_append (gsBuffer, fileitem);
	  bbrb_add_filename (gsBuffer->str);
	}
    }
  g_string_free (gsBufferCurrentPath, TRUE);
  g_string_free (gsBuffer, TRUE);
  filecancel (NULL, NULL);
}

void
PictureList_add (void)
{
  GString *gsbbrbrc;
  gchar arg[1024]="";
  gsbbrbrc=g_string_new(getenv("HOME"));
  g_string_append(gsbbrbrc,"/.bbrbrc");
  getarg(gsbbrbrc->str,"Path",arg);
  g_string_free(gsbbrbrc,TRUE);
  wpFileThumb = file_thumb_new ("Select a background image...");
  if(arg)
		file_thumb_set_filename(FILE_THUMB (wpFileThumb),arg);
  file_thumb_hide_fileop_buttons (FILE_THUMB (wpFileThumb));
  gtk_widget_hide (FILE_THUMB (wpFileThumb)->history_pulldown);
  gtk_signal_connect (GTK_OBJECT (FILE_THUMB (wpFileThumb)->loadall_button),
		      "clicked", (GtkSignalFunc) PictureList_add_loadall,
		      NULL);
  gtk_signal_connect (GTK_OBJECT (wpFileThumb), "destroy",
		      (GtkSignalFunc) filecancel, NULL);
  gtk_signal_connect (GTK_OBJECT (FILE_THUMB (wpFileThumb)->ok_button),
		      "clicked", (GtkSignalFunc) file_ok_sel, wpFileThumb);
  gtk_signal_connect_object (GTK_OBJECT
			     (FILE_THUMB (wpFileThumb)->cancel_button),
			     "clicked", (GtkSignalFunc) filecancel, NULL);
  gtk_widget_show (wpFileThumb);
  gtk_grab_add (wpFileThumb);
}

gchar *
PictureList_getRandom (void)
{
  GString *gsRandomPic;
  time_t *t;
  gint i = 0;
  t = malloc (sizeof(time_t));
  srand ((unsigned int) time (t));
  free (t);
  i = (int) ((0.0 + giRows) * rand () / (RAND_MAX + 1.0));
  gsRandomPic = g_string_new (PictureList_getRootCommand (i));
  g_string_assign (gsRandomPicStripped, "");
  for (i = 0; i < gsRandomPic->len; i++)
    {
      if (gsRandomPic->str[i] != '\\')
	g_string_append_c (gsRandomPicStripped, gsRandomPic->str[i]);
    }
  g_string_free (gsRandomPic, TRUE);
  return gsRandomPicStripped->str;
}
