/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
 * 
 * Further modified to support thumbnailed-masked files images by
 * Stacey Keast 23/09/1999 - 2001
 */

#ifndef FILETHUMB_H
#define FILETHUMB_H


#include <gdk_imlib.h>
#include <gdk/gdk.h>
#include <gtk/gtkwindow.h>
#include "textfuncs.h"
#include "thumbs.h"

#define TYPE_FILE_THUMB                    (file_thumb_get_type ())
#define FILE_THUMB(obj)                    (GTK_CHECK_CAST ((obj), TYPE_FILE_THUMB, FileThumb))
#define FILE_THUMB_CLASS(klass)            (GTK_CHECK_CLASS_CAST ((klass), TYPE_FILE_THUMB, FileThumbClass))
#define IS_FILE_THUMB(obj)                 (GTK_CHECK_TYPE ((obj), TYPE_FILE_THUMB))
#define IS_FILE_THUMB_CLASS(klass)         (GTK_CHECK_CLASS_TYPE ((klass), TYPE_FILE_THUMB))


typedef struct _FileThumb FileThumb;
typedef struct _FileThumbClass FileThumbClass;


struct _FileThumb
{
  GtkWindow window;

  GtkWidget *dir_list;
  GtkWidget *file_list;
  GtkWidget *selection_entry;
  GtkWidget *selection_text;
  GtkWidget *main_vbox;
  GtkWidget *ok_button;
  GtkWidget *cancel_button;
  GtkWidget *loadall_button;
  GtkWidget *help_button;
  GtkWidget *history_pulldown;
  GtkWidget *history_menu;
  GList *history_list;
  GtkWidget *fileop_dialog;
  GtkWidget *fileop_entry;
  gchar *fileop_file;
  gpointer cmpl_state;

  GtkWidget *fileop_c_dir;
  GtkWidget *fileop_del_file;
  GtkWidget *fileop_ren_file;

  GtkWidget *button_area;
  GtkWidget *action_area;

  GtkWidget *scrolled_win_files;
  GtkWidget *list_hbox;
  guint last_file_row;
  GtkWidget *wpPixmap;
};

struct _FileThumbClass
{
  GtkWindowClass parent_class;
};


GtkType file_thumb_get_type (void);
GtkWidget *file_thumb_new (const gchar * title);
void file_thumb_set_filename (FileThumb * filesel, const gchar * filename);
gchar *file_thumb_get_filename (FileThumb * filesel);
void file_thumb_complete (FileThumb * filesel, const gchar * pattern);
void file_thumb_show_fileop_buttons (FileThumb * filesel);
void file_thumb_hide_fileop_buttons (FileThumb * filesel);

#endif
